@@ -92,19 +92,70 @@ ciphersuite_strong = [cipher_AES256_SHA256, cipher_AES256_SHA1]
 ciphersuite_unencrypted :: [Cipher]
 ciphersuite_unencrypted = [cipher_null_MD5, cipher_null_SHA1]
 
+bulk_null = Bulk
+	{ bulkName         = "null"
+	, bulkKeySize      = 0
+	, bulkIVSize       = 0
+	, bulkBlockSize    = 0
+	, bulkF            = BulkNoneF
+	}
+
+bulk_rc4 = Bulk
+	{ bulkName         = "RC4-128"
+	, bulkKeySize      = 16
+	, bulkIVSize       = 0
+	, bulkBlockSize    = 0
+	, bulkF            = BulkStreamF initF_rc4 encryptF_rc4 decryptF_rc4
+	}
+
+bulk_aes128 = Bulk
+	{ bulkName         = "AES128"
+	, bulkKeySize      = 16
+	, bulkIVSize       = 16
+	, bulkBlockSize    = 16
+	, bulkF            = BulkBlockF aes128_cbc_encrypt aes128_cbc_decrypt
+	}
+
+bulk_aes256 = Bulk
+	{ bulkName         = "AES256"
+	, bulkKeySize      = 32
+	, bulkIVSize       = 16
+	, bulkBlockSize    = 16
+	, bulkF            = BulkBlockF aes256_cbc_encrypt aes256_cbc_decrypt
+	}
+
+hash_md5 = Hash
+	{ hashName = "MD5"
+	, hashSize = 16
+	, hashF    = MD5.hash
+	}
+
+hash_sha1 = Hash
+	{ hashName = "SHA1"
+	, hashSize = 20
+	, hashF    = SHA1.hash
+	}
+
+hash_sha256 = Hash
+	{ hashName = "SHA256"
+	, hashSize = 32
+	, hashF    = SHA256.hash
+	}
+
+hash_null = Hash
+	{ hashName = "null"
+	, hashSize = 0
+	, hashF    = const B.empty
+	}
+
 -- | this is not stricly a usable cipher; it's the initial cipher of a TLS connection
 cipher_null_null :: Cipher
 cipher_null_null = Cipher
 	{ cipherID           = 0x0
 	, cipherName         = "null-null"
-	, cipherDigestSize   = 0
-	, cipherKeySize      = 0
-	, cipherIVSize       = 0
-	, cipherKeyBlockSize = 0
-	, cipherPaddingSize  = 0
-	, cipherMACHash      = (const B.empty)
+	, cipherBulk         = bulk_null
+	, cipherHash         = hash_null
 	, cipherKeyExchange  = CipherKeyExchange_RSA
-	, cipherF            = CipherNoneF
 	, cipherMinVer       = Nothing
 	}
 
@@ -113,14 +164,9 @@ cipher_null_MD5 :: Cipher
 cipher_null_MD5 = Cipher
 	{ cipherID           = 0x1
 	, cipherName         = "RSA-null-MD5"
-	, cipherDigestSize   = 16
-	, cipherKeySize      = 0
-	, cipherIVSize       = 0
-	, cipherKeyBlockSize = 2 * (16 + 0 + 0)
-	, cipherPaddingSize  = 0
-	, cipherMACHash      = MD5.hash
+	, cipherBulk         = bulk_null
+	, cipherHash         = hash_md5
 	, cipherKeyExchange  = CipherKeyExchange_RSA
-	, cipherF            = CipherNoneF
 	, cipherMinVer       = Nothing
 	}
 
@@ -129,14 +175,9 @@ cipher_null_SHA1 :: Cipher
 cipher_null_SHA1 = Cipher
 	{ cipherID           = 0x2
 	, cipherName         = "RSA-null-SHA1"
-	, cipherDigestSize   = 20
-	, cipherKeySize      = 0
-	, cipherIVSize       = 0
-	, cipherKeyBlockSize = 2 * (20 + 0 + 0)
-	, cipherPaddingSize  = 0
-	, cipherMACHash      = SHA1.hash
+	, cipherBulk         = bulk_null
+	, cipherHash         = hash_sha1
 	, cipherKeyExchange  = CipherKeyExchange_RSA
-	, cipherF            = CipherNoneF
 	, cipherMinVer       = Nothing
 	}
 
@@ -145,14 +186,9 @@ cipher_RC4_128_MD5 :: Cipher
 cipher_RC4_128_MD5 = Cipher
 	{ cipherID           = 0x04
 	, cipherName         = "RSA-rc4-128-md5"
-	, cipherDigestSize   = 16
-	, cipherKeySize      = 16
-	, cipherIVSize       = 0
-	, cipherKeyBlockSize = 2 * (16 + 16 + 0)
-	, cipherPaddingSize  = 0
-	, cipherMACHash      = MD5.hash
+	, cipherBulk         = bulk_rc4
+	, cipherHash         = hash_md5
 	, cipherKeyExchange  = CipherKeyExchange_RSA
-	, cipherF            = CipherStreamF initF_rc4 encryptF_rc4 decryptF_rc4
 	, cipherMinVer       = Nothing
 	}
 
@@ -161,14 +197,9 @@ cipher_RC4_128_SHA1 :: Cipher
 cipher_RC4_128_SHA1 = Cipher
 	{ cipherID           = 0x05
 	, cipherName         = "RSA-rc4-128-sha1"
-	, cipherDigestSize   = 20
-	, cipherKeySize      = 16
-	, cipherIVSize       = 0
-	, cipherKeyBlockSize = 2 * (20 + 16 + 0)
-	, cipherPaddingSize  = 0
-	, cipherMACHash      = SHA1.hash
+	, cipherBulk         = bulk_rc4
+	, cipherHash         = hash_sha1
 	, cipherKeyExchange  = CipherKeyExchange_RSA
-	, cipherF            = CipherStreamF initF_rc4 encryptF_rc4 decryptF_rc4
 	, cipherMinVer       = Nothing
 	}
 
@@ -177,14 +208,9 @@ cipher_AES128_SHA1 :: Cipher
 cipher_AES128_SHA1 = Cipher
 	{ cipherID           = 0x2f
 	, cipherName         = "RSA-aes128-sha1"
-	, cipherDigestSize   = 20
-	, cipherKeySize      = 16
-	, cipherIVSize       = 16
-	, cipherKeyBlockSize = 2 * (20 + 16 + 16)
-	, cipherPaddingSize  = 16
-	, cipherMACHash      = SHA1.hash
+	, cipherBulk         = bulk_aes128
+	, cipherHash         = hash_sha1
 	, cipherKeyExchange  = CipherKeyExchange_RSA
-	, cipherF            = CipherBlockF aes128_cbc_encrypt aes128_cbc_decrypt
 	, cipherMinVer       = Just SSL3
 	}
 
@@ -193,14 +219,9 @@ cipher_AES256_SHA1 :: Cipher
 cipher_AES256_SHA1 = Cipher
 	{ cipherID           = 0x35
 	, cipherName         = "RSA-aes256-sha1"
-	, cipherDigestSize   = 20
-	, cipherKeySize      = 32
-	, cipherIVSize       = 16
-	, cipherKeyBlockSize = 2 * (20 + 32 + 16)
-	, cipherPaddingSize  = 16
-	, cipherMACHash      = SHA1.hash
+	, cipherBulk         = bulk_aes256
+	, cipherHash         = hash_sha1
 	, cipherKeyExchange  = CipherKeyExchange_RSA
-	, cipherF            = CipherBlockF aes256_cbc_encrypt aes256_cbc_decrypt
 	, cipherMinVer       = Just SSL3
 	}
 
@@ -209,14 +230,9 @@ cipher_AES128_SHA256 :: Cipher
 cipher_AES128_SHA256 = Cipher
 	{ cipherID           = 0x3c
 	, cipherName         = "RSA-aes128-sha256"
-	, cipherDigestSize   = 32
-	, cipherKeySize      = 16
-	, cipherIVSize       = 16
-	, cipherKeyBlockSize = 2 * (32 + 16 + 16)
-	, cipherPaddingSize  = 16
-	, cipherMACHash      = SHA256.hash
+	, cipherBulk         = bulk_aes128
+	, cipherHash         = hash_sha256
 	, cipherKeyExchange  = CipherKeyExchange_RSA
-	, cipherF            = CipherBlockF aes128_cbc_encrypt aes128_cbc_decrypt
 	, cipherMinVer       = Just TLS12
 	}
 
@@ -225,14 +241,9 @@ cipher_AES256_SHA256 :: Cipher
 cipher_AES256_SHA256 = Cipher
 	{ cipherID           = 0x3d
 	, cipherName         = "RSA-aes256-sha256"
-	, cipherDigestSize   = 32
-	, cipherKeySize      = 32
-	, cipherIVSize       = 16
-	, cipherKeyBlockSize = 2 * (32 + 32 + 16)
-	, cipherPaddingSize  = 16
-	, cipherMACHash      = SHA256.hash
+	, cipherBulk         = bulk_aes256
+	, cipherHash         = hash_sha256
 	, cipherKeyExchange  = CipherKeyExchange_RSA
-	, cipherF            = CipherBlockF aes256_cbc_encrypt aes256_cbc_decrypt
 	, cipherMinVer       = Just TLS12
 	}
 
